from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponse
from .forms import ProductUploadForm
from django.contrib.auth.models import User, auth
from cart.models import PCart, Quantity, ContactCart, ContactList, CPCart
from accounts.models import SellerData, KisanData
from accounts.views import seller_login_required, kisan_login_required
from kisan.models import ProductData, Rating, UsersRated, Category, SubCategory
from django.db.models import Q
from django.contrib import messages
import os
from django.contrib.auth.decorators import login_required
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
import random

# Create your views here.
def s1(request):
    return render(request, 'seller/s1.html')


def cat_view(request, cat_name, cat_id):
    sub_result = list(SubCategory.objects.filter(category=cat_id))
    page = request.GET.get('page', 1)
    paginator = Paginator(sub_result, 15)
    try:
        subcat_list = paginator.page(page)
    except PageNotAnInteger:
        subcat_list = paginator.page(1)
    except EmptyPage:
        subcat_list = paginator.page(paginator.num_pages)
    return render(request, 'seller/cat_view.html', {'subcategory': subcat_list, 'cat_nm': cat_name})


def subcat_view(request, cat_name, subcat_name, subcat_id):
    prod_results = list(ProductData.objects.filter(subcategory_id=subcat_id))
    return render(request, 'seller/subcat_view.html', {'product': prod_results, 'subcat_nm': subcat_name})


@seller_login_required
def sel_deshboard(request):
    return render(request, 'seller/sel_deshboard.html')


@seller_login_required
def product_upload(request):
    b = list(SellerData.objects.filter(email=request.user.username).values_list('comp_name', flat=True))
    if request.method == 'POST':
        form = ProductUploadForm(request.POST, request.FILES)
        if form.is_valid():
            instance = form.save(commit=False)
            instance.seller_name = b[0]
            instance.p_owner = request.user
            instance.save();
            messages.info(request, 'Product uploaded successfully!')
            return redirect('/seller/my_prod')
        else:
            return render(request, 'seller/product_upload.html', {'form': form})
    else:
        form = ProductUploadForm()
    return render(request, 'seller/product_upload.html', {
        'form': form
    })


# @seller_login_required
# def profile(request):
#     x = SellerData.objects.all()
#     for y in x:
#         if y.email == request.user.username:
#             return render(request, 'seller/profile.html', {'data': y})
#     return render(request, 'seller/profile.html')

@seller_login_required
def profile(request):
    x = SellerData.objects.get(user=request.user)
    return render(request, 'seller/profile.html', {'data': x})


from accounts.forms import SellerProfileForm
@seller_login_required
def profile_edit(request):
    obj = get_object_or_404(SellerData, user=request.user)
    form = SellerProfileForm(request.POST or None, request.FILES or None, instance=obj)
    context = {'form': form}
    print(form.is_valid())
    if form.is_valid():
        print('valid')
        obj = form.save(commit=False)
        obj.save()
        messages.info(request, 'Profile updated successfully!')
        context = {'form': form}
        # return render(request, 'seller/product_upload.html', context)
        return redirect('/seller/profile/edit')

    else:
        context = {'form': form}
        # return render(request, 'seller/edit_prod.html', context)
        return render(request, 'seller/profile_edit.html', {'data': form})


def search(request):
    query = request.GET.get('q', None)
    x = ProductData.objects.none()

    if query is not None:
        lookups = Q(p_name__icontains=query) | Q(p_desc__icontains=query)
        x = ProductData.objects.filter(lookups).distinct()
        return render(request, 'customer/product_display.html', {'prods': x})
    return render(request, 'customer/product_display.html', {'prods': x})


def browse(request):
    p = ProductData.objects.all()
    sub_cat = SubCategory.objects.all()
    query_Set = Category.objects.filter(Q(is_active=True) & Q(cat_type='SP'))
    page = request.GET.get('page', 1)
    page1 = request.GET.get(1, 1)
    paginator = Paginator(query_Set, 3)
    paginator1 = Paginator(sub_cat, 5)
    sub_cat = paginator1.page(page1)
    try:
        cat = paginator.page(page)
    except PageNotAnInteger:
        cat = paginator.page(1)
    except EmptyPage:
        cat = paginator.page(paginator.num_pages)
    subcat1 = []
    try:
        for obj in cat:
            query_Set = SubCategory.objects.filter(Q(category=obj) | Q(category__cat_type='KP'))
            page1 = request.GET.get(1, 1)
            paginator1 = Paginator(query_Set, 5)
            sub_cat = paginator1.page(page1)
            subcat1+=sub_cat
            random.shuffle(subcat1)
    except:
        pass
    return render(request, 'seller/products_display.html', {'prods': p, 'categories': cat, 'sub_category': subcat1})


@seller_login_required
def my_prod(request):
    # s_email = request.user.username
    # p = ProductData.objects.filter(s_username__exact=s_email)
    p = ProductData.objects.filter(p_owner=request.user)
    total = len(list(p.values()))
    return render(request, 'seller/my_prod.html', {'prods': p, 'total': total})


@seller_login_required
def editpage(request, object_id):
    obj = get_object_or_404(ProductData, id=object_id)
    form = ProductUploadForm(request.POST or None, request.FILES or None, instance=obj)
    context = {'form': form}

    if request.method == 'POST' and 'deleter' in request.POST:
        file = obj.p_img.path
        ProductData.objects.filter(id=object_id).delete()
        os.remove(file)
        messages.info(request, 'Product Deleted')
        return redirect('/seller/my_prod')

    if form.is_valid():
        obj = form.save(commit=False)
        obj.save()
        messages.info(request, 'Product updated successfully!')
        context = {'form': form}
        # return render(request, 'seller/product_upload.html', context)
        return redirect('/seller/my_prod')

    else:
        context = {'form': form}
        return render(request, 'seller/edit_prod.html', context)

@seller_login_required
def delete_record(request, object_id):
    obj = get_object_or_404(ProductData, id=object_id)
    file = obj.p_img.path
    ProductData.objects.filter(id=object_id).delete()
    os.remove(file)
    messages.info(request, 'Product Deleted')
    return redirect('/seller')


def desc(request, object_id):
    product = ProductData.objects.get(id=object_id)
    sellers = SellerData.objects.all()
    list_products = ProductData.objects.filter(p_name__icontains=product.p_name).order_by('p_price')
    p1 = ProductData.objects.filter(id=list_products[0].id).values()
    s = p1[0]['seller_name']
    num_res1 = Rating.objects.filter(seller=s).count()
    if num_res1 == 0:
        co = 0
        us = []
    else:
        co = Rating.objects.filter(seller=s).values()[0]['rate']
        us1 = Rating.objects.get(seller=s)
        us = us1.users.values_list()
    if (request.method == 'POST' or 'deleter' in request.POST) and request.user.is_authenticated:
        Q = request.POST.get('quant', list_products[0].min_q)
        # user_ins = User.objects.get(username=request.user.username)
        user_ins = request.user
        num_res = CPCart.objects.filter(user=user_ins).count()
        if num_res == 0:
            p = CPCart(user=user_ins)
            p.save()
        elif num_res == 1:
            print('object exist need to update')
            # print(PCart.objects.filter(user=request.user.username, quant=Quantity.objects.filter(min_q=2))).values()
        q = Quantity(min_q=Q)
        # print(q)
        q.save()
        p1 = CPCart.objects.get(user=user_ins)
        p1.p_name.add(list_products[0])
        p1.quant.add(q)
        p1.save()
    else:
        if request.method == 'POST':
            messages.info(request, 'Please login to save product')
    return render(request, 'seller/product_description.html',
                  {'data': product, 'listdata': list_products, 't': 0, 'co': co, 'us': us, 'seller': sellers})


def desc1(request, object_id):
    product = ProductData.objects.get(id=object_id)
    sellers = SellerData.objects.all()
    list_products = ProductData.objects.filter(p_name__icontains=product.p_name).order_by('p_price')
    p1 = ProductData.objects.filter(id=product.id).values()
    s = p1[0]['seller_name']
    num_res1 = Rating.objects.filter(seller=s).count()
    if num_res1 == 0:
        co = 0
    else:
        co = Rating.objects.filter(seller=s).values()[0]['rate']
    if (request.method == 'POST' or 'deleter' in request.POST) and request.user.is_authenticated:
        Q = request.POST.get('quant', product.min_q)
        # user_ins = User.objects.get(username=request.user.username)
        user_ins = request.user
        num_res = CPCart.objects.filter(user=user_ins).count()
        if num_res == 0:
            p = CPCart(user=user_ins)
            p.save()
        elif num_res > 1:
            print('object exist need to update')

        q = Quantity(min_q=Q)
        # print(q)
        q.save()
        p1 = CPCart.objects.get(user=user_ins)
        p1.p_name.add(product)
        p1.quant.add(q)
        p1.save()
    else:
        if request.method == 'POST':
            messages.info(request, 'Please login to save product')
    return render(request, 'seller/product_description.html',
                  {'data': product, 'listdata': list_products, 't': 1, 'co': co, 'seller':sellers})


def get_subcategory(request):
    id = request.GET.get('id', '')
    result = list(SubCategory.objects.filter(category_id=int(id)).values('id', 'cat_title'))
    return HttpResponse(json.dumps(result), content_type="application/json")

def categories_m(request):
    query_Set = Category.objects.all()
    page = request.GET.get('page', 1)
    paginator = Paginator(query_Set, 20)
    try:
        cat = paginator.page(page)
    except PageNotAnInteger:
        cat = paginator.page(1)
    except EmptyPage:
        cat = paginator.page(paginator.num_pages)
    return render(request, 'index.html', {'categories': cat})


def subcategory_product(request, object_id):
    query_Set = ProductData.objects.filter(subcategory=object_id).order_by('p_price')
    subcat_des = SubCategory.objects.get(id=object_id)
    sub_cat = SubCategory.objects.all()
    cat = Category.objects.all()
    page = request.GET.get('page', 1)
    paginator = Paginator(query_Set, 5)
    try:
        prod = paginator.page(page)
    except PageNotAnInteger:
        prod = paginator.page(1)
    except EmptyPage:
        prod = paginator.page(paginator.num_pages)
    return render(request, 'seller/subcat_products_display.html', {'prods': prod, 'categories': cat, 'sub_category': sub_cat, 'subcat_des': subcat_des})

@login_required
def seller_details(request, object_id, object_name):
    user_id = ProductData.objects.filter(id=object_id).values_list('p_owner__username', flat=True)
    # seller = SellerData.objects.get(id=object_id, name=object_name)
    # contact_details = KisanData.objects.get(name=object_name)
    print(user_id[0])
    # contact_details = SellerData.objects.get(email=user_id[0])
    if KisanData.objects.filter(mobile=user_id[0]):
        contact_details = KisanData.objects.get(mobile=user_id[0])
    else:
        contact_details = SellerData.objects.get(email=user_id[0])
    print(contact_details)
    # print(contact_details.name)
    if (request.method == 'POST' or 'deleter' in request.POST) and request.user.is_authenticated:
        id = str(request.POST.get('seller_id'))
        name = str(request.POST.get('seller_name'))
        phone = int(request.POST.get('seller_mobile'))
        # print(name, phone)
        p = ContactCart.objects.get_or_create(user=request.user.username)
        # print(request.user.email)
        p1 = ContactList.objects.filter(s_name=name, s_mobile=phone)
        # print(p1.values())
        if p1==True:
            p2 = ContactList.objects.create(s_id=id, s_name=name, s_mobile=phone)
            # print(p2)

            p[0].contact_list.add(p2)
            messages.info(request, 'Added in contact list')
            # return redirect('/')
        else:
            p2 = ContactList.objects.create(s_name=name, s_mobile=phone)
            # print(p2)
            p[0].contact_list.add(p2)
            messages.info(request, 'Contact already exist')
            # return redirect('/')
    num_res1 = Rating.objects.filter(seller=object_name).count()
    if num_res1 == 0:
        co = 0
    else:
        co = Rating.objects.filter(seller=object_name).values()[0]['rate']
    return render(request, 'home/contact_detail.html',
                  {'contact_details': contact_details, 'co': co})

